@@ -252,7 +252,7 @@ class gen_gui_test_case(_GuiTestCase):
                     await self.page.wait_for_selector(nth_selector, state='attached',
                                                       timeout=max_time)
                     found = True
-                except:
+                except Exception:
                     num_tries += 1
 
             num_tries = 0
@@ -263,7 +263,7 @@ class gen_gui_test_case(_GuiTestCase):
                     await self.page.wait_for_selector(nth_selector, state='detached',
                                                       timeout=max_time)
                     found = True
-                except:
+                except Exception:
                     num_tries += 1
 
         else:
@@ -275,7 +275,7 @@ class gen_gui_test_case(_GuiTestCase):
                     await self.page.wait_for_selector(nth_selector, state='detached',
                                                       timeout=max_time)
                     found = True
-                except:
+                except Exception:
                     num_tries += 1
 
         hndl_list = await self.page.query_selector_all(selector)
