@@ -8,9 +8,6 @@ import types
 import base64
 import zlib
 
-from shutil import rmtree
-from tempfile import mkdtemp
-
 import numpy as np
 
 import openmdao.api as om
@@ -101,7 +98,7 @@ class TestViewerData(unittest.TestCase):
                     val = np.asarray(tree[key])
                     try:
                         assert_near_equal(expected_val, val, tolerance=val_tol)
-                    except Exception as e:
+                    except Exception:
                         raise AssertionError(f'{_path + [key]} did not match to expected value')
 
         for i, key in enumerate(expected_tree.keys()):
@@ -306,7 +303,7 @@ class TestViewerData(unittest.TestCase):
         self.check_viewer_data(_get_viewer_data(p), 'sellar_initial_values.json')
 
         # recorded viewer data should match
-        self.check_viewer_data(_get_viewer_data(filename), 'sellar_initial_values.json')
+        self.check_viewer_data(_get_viewer_data(p.get_outputs_dir() / filename), 'sellar_initial_values.json')
 
         # there should be final values when data is generated after run_model
         p.run_model()
@@ -347,6 +344,9 @@ class TestViewerData(unittest.TestCase):
         prob.setup()
         prob.final_setup()
 
+        # Uncomment to update regression data
+        # save_viewer_data(_get_viewer_data(prob), 'betz_tree.json')
+
         # check expected viewer data
         self.check_viewer_data(_get_viewer_data(prob), 'betz_tree.json')
 
@@ -391,6 +391,7 @@ class TestViewerData(unittest.TestCase):
         prob = om.Problem()
         prob.model.add_subsystem('comp', SystemWithNdArrayOption(arr=np.ones(2)))
         prob.setup()
+        prob.final_setup()
 
         viewer_data = _get_viewer_data(prob)
         np.testing.assert_equal(viewer_data['tree']['children'][1]['options']['arr'],
@@ -413,6 +414,7 @@ class TestViewerData(unittest.TestCase):
         comp = prob.model.add_subsystem('comp', SystemWithLargeOption())
         comp.options['large_option'] = np.zeros(int(1e4))
         prob.setup()
+        prob.final_setup()
 
         viewer_data = _get_viewer_data(prob)
         self.assertEqual(viewer_data['tree']['children'][1]['options']['large_option'],
@@ -459,7 +461,7 @@ class TestViewerData(unittest.TestCase):
             subprob_data = extract_compressed_model('N2subprob.html')
 
             # check problem data generated from recording against data generated from problem
-            check_call(f"openmdao n2 {sql_filename} -o N2recording.html"
+            check_call(f"openmdao n2 {p.get_outputs_dir() / sql_filename} -o N2recording.html"
                        f"{' --no_browser' if not DEBUG_BROWSER else ''}")
             recording_data = extract_compressed_model('N2recording.html')
 
@@ -480,17 +482,9 @@ class TestViewerData(unittest.TestCase):
             p.model.add_subsystem('submodelcomp',
                                 om.SubmodelComp(problem=subprob, inputs=['*'], outputs=['*'], do_coloring=False),
                                 promotes=['*'])
-            p.model.add_subsystem('supercomp',
-                                om.ExecComp('z = 3 * y'),
-                                promotes=['*'])
+            p.model.add_subsystem('supercomp', om.ExecComp('z = 3 * y'), promotes=['*'])
 
             p.setup()
-            p.final_setup()
-
-            # verify model runs correctly
-            p.set_val('x1', 1)
-            p.set_val('x2', 2)
-            p.set_val('x3', 3)
 
             p.run_model()
             """
@@ -588,12 +582,14 @@ class TestN2(unittest.TestCase):
                 n2_from_prob_html = f"n2_from_prob_{values}.html"
                 n2_from_file_html = f"n2_from_file_{values}.html"
 
+                _sql_filename = p.get_outputs_dir() / sql_filename
+
                 if values is not None:
                     n2(p, outfile=n2_from_prob_html, show_browser=DEBUG_BROWSER, values=values)
-                    n2(sql_filename, outfile=n2_from_file_html, show_browser=DEBUG_BROWSER, values=values)
+                    n2(_sql_filename, outfile=n2_from_file_html, show_browser=DEBUG_BROWSER, values=values)
                 else:
                     n2(p, outfile=n2_from_prob_html, show_browser=DEBUG_BROWSER)
-                    n2(sql_filename, outfile=n2_from_file_html, show_browser=DEBUG_BROWSER)
+                    n2(_sql_filename, outfile=n2_from_file_html, show_browser=DEBUG_BROWSER)
 
                 # Compare models from the files generated from the Problem and the recording
                 model_data_from_prob = extract_compressed_model(n2_from_prob_html)
@@ -604,9 +600,9 @@ class TestN2(unittest.TestCase):
                 # also check data generated using n2 command
                 n2_from_cmd_html = f"n2_from_cmd_{values}.html"
                 if values is not False:
-                    cmd = f"openmdao n2 --no_browser -o {n2_from_cmd_html} {sql_filename}"
+                    cmd = f"openmdao n2 --no_browser -o {n2_from_cmd_html} {_sql_filename}"
                 else:
-                    cmd = f"openmdao n2 --no_values --no_browser  -o {n2_from_cmd_html} {sql_filename}"
+                    cmd = f"openmdao n2 --no_values --no_browser  -o {n2_from_cmd_html} {_sql_filename}"
                 check_call(cmd)
 
                 model_data_from_cmd = extract_compressed_model(n2_from_cmd_html)
@@ -659,6 +655,7 @@ class TestN2(unittest.TestCase):
 
         with self.assertRaises(Exception) as cm:
             p.setup()
+            p.final_setup()
 
         self.assertEqual(cm.exception.args[0], msg)
 
