@@ -560,6 +560,9 @@ def _run_scaling_report(driver, report_filename=_default_scaling_filename):
     prob = driver._problem()
     scaling_filepath = prob.get_reports_dir() / report_filename
 
+    if not driver.supports['optimization']:
+        return
+
     try:
         prob.driver.scaling_report(outfile=scaling_filepath, show_browser=False)
 
